<?php

class LAGatsbyMLSettingsProxy {

    private $settings;
    private $type_override;

    public function __construct($settings, $override_type = null) {
        $this->settings = $settings;
        $this->type_override = $override_type;
    }

    public function getType(){
        $current_type = $this -> settings['nksub_mail_type'];
        if(!is_null($this->type_override)) $current_type = $this->type_override;

        switch($current_type){
            case 'email' :
                $type = 'my-email';
                break;
            case 'mc' :
                $type = 'mailchimp';
                break;
            case 'gr' :
                $type = 'getresponse';
                break;
            case 'cc' :
                $type = 'constantcontact';
                break;
            case 'ic' :
                $type = 'icontact';
                break;
            case 'bm' :
                $type = 'benchmarkmail';
                break;
            case 'mm' :
                $type = 'mymail';
                break;
            case 'aw' :
                $type = 'aweber';
                break;
            case 'mp' :
                $type = 'mailpoet';
                break;
            case 'mr' :
                $type = 'mailrelay';
                break;
            case 'eg' :
                $type = 'e-goi';
                break;
            case 'mmi' :
                $type = 'madmimi';
                break;
            case 'cm' :
                $type = 'campaignmonitor';
                break;
            case 'fm' :
                $type = 'freshmail';
                break;
        }
        return $type;
    }

    public function getSettings(){
        switch($this -> getType()){
            case 'my-email':
                $settings = array(
                    'my-email' => $this -> settings['nksub_email']
                );

                break;
            case 'mailchimp':
                $settings = array(
                    'mailchimp-api-key' => $this -> settings['nksub_mc_key'],
                    'mailchimp-list-id' => $this -> settings['nksub_mc_list_id']
                );

                break;
            case 'getresponse':
                $settings = array(
                    'getresponse-api-key' => $this -> settings['nksub_gr_key'],
                    'getresponse-list-id' => $this -> settings['nksub_gr_list_id']
                );
                break;
            case 'constantcontact':
                $settings = array(
                    'constantcontact-username' => $this -> settings['nksub_cc_user'],
                    'constantcontact-password' => $this -> settings['nksub_cc_pass'],
                    'constantcontact-list-id' => $this -> settings['nksub_cc_list_id']
                );

                break;
            case 'icontact':
                $settings = array(
                    'icontact-api-app-id' => $this -> settings['nksub_ic_app'],
                    'icontact-api-username' => $this -> settings['nksub_ic_user'],
                    'icontact-api-password' => $this -> settings['nksub_ic_pass'],
                    'icontact-list-id' => $this -> settings['nksub_ic_list_id']
                );

                break;
            case 'benchmarkmail':
                $settings = array(
                    'benchmarkmail-username' => $this -> settings['nksub_bm_user'],
                    'benchmarkmail-password' => $this -> settings['nksub_bm_pass'],
                    'benchmarkmail-list-id' => $this -> settings['nksub_bm_list_id']
                );

                break;
            case 'mymail':
                $settings = array(
                    'mymail-list-id' => $this -> settings['nksub_mm_list_id']
                );

                break;
            case 'aweber':
                $settings = array(
                    'aweber-consumer-key' => $this -> settings['nksub_aw_consumer_key'],
                    'aweber-consumer-secret' => $this -> settings['nksub_aw_consumer_secret'],
                    'aweber-access-key' => $this -> settings['nksub_aw_access_key'],
                    'aweber-access-secret' => $this -> settings['nksub_aw_access_secret'],
                    'aweber-list-id' => $this -> settings['nksub_aw_list_id']
                );

                break;
            case 'mailpoet':
                $settings = array(
                    'mailpoet-list-id' => $this -> settings['nksub_mp_list_id']
                );

                break;
            case 'mailrelay':
                $settings = array(
                    'mailrelay-api-host' => $this -> settings['nksub_mr_api_host'],
                    'mailrelay-api-key' => $this -> settings['nksub_mr_api_key'],
                    'mailrelay-list-id' => $this -> settings['nksub_mr_list_id']
                );
                break;
            case 'e-goi':
                $settings = array(
                    'e-goi-api-key' => $this -> settings['nksub_eg_api_key'],
                    'e-goi-list-id' => $this -> settings['nksub_eg_list_id']
                );
                break;
            case 'madmimi':
                $settings = array(
                    'madmimi-api-username' => $this -> settings['nksub_mmi_api_username'],
                    'madmimi-api-key' => $this -> settings['nksub_mmi_api_key'],
                    'madmimi-list-id' => $this -> settings['nksub_mmi_list_id']
                );
                break;
            case 'campaignmonitor':
                $settings = array(
                    'campaignmonitor-api-client-id' => $this -> settings['nksub_cm_api_client_id'],
                    'campaignmonitor-api-key' => $this -> settings['nksub_cm_api_key'],
                    'campaignmonitor-list-id' => $this -> settings['nksub_cm_list_id']
                );
                break;
            case 'freshmail':
                $settings = array(
                    'freshmail-api-key' => $this -> settings['nksub_fm_api_key'],
                    'freshmail-api-secret' => $this -> settings['nksub_fm_api_secret'],
                    'freshmail-list-id' => $this -> settings['nksub_fm_list_id']
                );
                break;
        }

        return $settings;
    }
}
?>