<?php
    if (!class_exists('LAGatsbyMailingListWrapper')) {

        if(!class_exists('LA_Mailchimp')) include_once('vendor/Mailchimp.php');
        include_once('vendor/getresponse/jsonRPCClient.php'); # condition inside of a class
        include_once('vendor/constantcontact/class.cc.php'); # condition inside of a class
        if(!class_exists('iContactApi')) include_once('vendor/icontact-api-php/lib/iContactApi.php');
        if(!class_exists('BMEAPI')) include_once('vendor/benchmarkmail/BMEAPI.class.php');
        if(!class_exists('AWeberAPI')) include_once('vendor/aweber_api/aweber.php');
        if(!class_exists('LAMailRelay')) include_once('vendor/la_mailrelay.php');
        if(!class_exists('EgoiApiFactory')) include_once('vendor/e-goi/Egoi/Factory.php');
        if(!class_exists('MadMimi')) include_once('vendor/madmimi/MadMimi.class.php');
        if(!class_exists('CS_REST_Clients')) include_once('vendor/campaignmonitor/csrest_clients.php');
        if(!class_exists('CS_REST_Subscribers')) include_once('vendor/campaignmonitor/csrest_subscribers.php');
        if(!class_exists('FmRestApi')) include_once('vendor/freshmail.wrapper.php');


        class LAGatsbyMailingListWrapper
        {

            const BENCHMARKMAIL_API_URL = 'http://www.benchmarkemail.com/api/1.3';
            const ALREADY_SUBSCRIBED_CODE = 'already_subscribed';
            const EGOI_PLUGIN_KEY = '607ef76d3b191f235b3cf9dd4916036a';

            private $ml_type;
            private $ml_settings;
            private $ml_last_error;

            function __construct($ml_type, $ml_settings)
            {
                $this->ml_settings = $ml_settings;
                $this->ml_type = $ml_type;
            }

            public function getLastError(){ return $this->ml_last_error; }

            public function getMailingLists()
            {
                try {
                    switch ($this->ml_type) {
                        case 'mailchimp':
                            $response = $this->getMailchimpMailingLists();
                            break;
                        case 'getresponse':
                            $response = $this->getGetResponseMailingLists();
                            break;
                        case 'constantcontact':
                            $response = $this->getConstantContactMailingLists();
                            break;
                        case 'icontact':
                            $response = $this->getIContactMailingLists();
                            break;
                        case 'benchmarkmail':
                            $response = $this->getBenchmarkMailMailingLists();
                            break;
                        case 'mymail':
                            $response = $this->getMyMailMailingLists();
                            break;
                        case 'aweber':
                            $response = $this->getAWeberMailingLists();
                            break;
                        case 'mailpoet':
                            $response = $this->getMailPoetMailingLists();
                            break;
                        case 'mailrelay':
                            $response = $this->getMailRelayMailingLists();
                            break;
                        case 'e-goi':
                            $response = $this->getEGoiMailingLists();
                            break;
                        case 'madmimi':
                            $response = $this->getMadMimiMailingLists();
                            break;
                        case 'campaignmonitor':
                            $response = $this->getCampaignMonitorMailingLists();
                            break;
                        case 'freshmail':
                            $response = $this->getFreshmailMailingLists();
                            break;
                        default:
                            $response = array(1);
                    }
                } catch (Exception $e) {
                    $response = array();
                    print_r("<!-- NINJA ERROR \n\n\n".var_export($e, true)."\n\n\n-->");
                }
                return $response;
            }

            private function getMailchimpMailingLists()
            {
                if(!$this -> validateSettings(array('mailchimp-api-key'))) return array();
                $mc = new LA_Mailchimp($this->ml_settings['mailchimp-api-key']);
                $mailLists = array();
                $lists = $mc->get('lists');
                //nksub_debug_to_console($lists);
                foreach ($lists['lists'] as $listItem) {
                    array_push($mailLists, array($listItem['id'], $this->sanitizeName($listItem['name'])));
                }
                return $mailLists;
            }

            private function getGetResponseMailingLists()
            {
                if(!$this -> validateSettings(array('getresponse-api-key'))) return array();
                $client = new jsonRPCClient('http://api2.getresponse.com');

                $mailLists = array();

                $result = $client->get_campaigns($this->ml_settings['getresponse-api-key']);

                foreach ($result as $id => $listItem) {
                    array_push($mailLists, array($id, $this->sanitizeName($listItem['name'])));
                }

                return $mailLists;
            }

            private function getConstantContactMailingLists()
            {
                if(!$this -> validateSettings(array('constantcontact-username', 'constantcontact-password'))) return array();

                $client = new cc($this->ml_settings['constantcontact-username'], $this->ml_settings['constantcontact-password']);

                $mailLists = array();

                $lists = $client->get_lists();

                if (is_array($lists) || is_object($lists))
                {
                    foreach ($lists as $listItem) {
                        array_push($mailLists, array($listItem['id'], $this->sanitizeName($listItem['Name'])));
                    }
                }

                return $mailLists;
            }

            private function getIContactMailingLists()
            {
                if(!$this -> validateSettings(array('icontact-api-app-id', 'icontact-api-password', 'icontact-api-username'))) return array();

                $oiContact = iContactApi::getInstance()->setConfig(array(
                    'appId' => $this->ml_settings['icontact-api-app-id'],
                    'apiPassword' => $this->ml_settings['icontact-api-password'],
                    'apiUsername' => $this->ml_settings['icontact-api-username']
                ));

                $mailLists = array();

                foreach ($oiContact->getLists() as $listItem) {
                    array_push($mailLists, array($listItem->listId, $this->sanitizeName($listItem->name)));
                }
                return $mailLists;
            }

            private function getBenchmarkMailMailingLists()
            {
                if(!$this -> validateSettings(array('benchmarkmail-username', 'benchmarkmail-password'))) return false;

                $bmApi = new BMEAPI($this->ml_settings['benchmarkmail-username'], $this->ml_settings['benchmarkmail-password'], LAGatsbyMailingListWrapper::BENCHMARKMAIL_API_URL);

                $mailLists = array();

                $resp = $bmApi->listGet("", 1, 50, "", "");

                if($resp){
                    foreach ($resp as $listItem) {
                        array_push($mailLists, array($listItem['id'], $this->sanitizeName($listItem['listname'])));
                    }
                }

                return $mailLists;
            }

            private function getMyMailMailingLists()
            {
                if(!function_exists('mymail')) return false;

                $mailLists = array();

                $lists = mymail('lists')->get();

                foreach ($lists as $listItem) {
                    array_push($mailLists, array($listItem -> ID, $this->sanitizeName($listItem -> name)));
                }

                return $mailLists;
            }

            private function getAWeberMailingLists()
            {
                if(!$this -> validateSettings(array('aweber-consumer-key','aweber-consumer-secret','aweber-access-key', 'aweber-access-secret'))) return false;
                $mailLists = array();

                try {
                    $consumerKey = $this->ml_settings['aweber-consumer-key'];
                    $consumerSecret = $this->ml_settings['aweber-consumer-secret'];
                    $accessKey = $this->ml_settings['aweber-access-key'];
                    $accessSecret = $this->ml_settings['aweber-access-secret'];

                    $aweber = new AWeberAPI($consumerKey, $consumerSecret);

                    $account = $aweber->getAccount($accessKey, $accessSecret);

                    $resp = $account->lists;

                    if(count($resp)){
                        foreach ($resp as $listItem) {
                            array_push($mailLists, array($listItem -> id, $this->sanitizeName($listItem->name)));
                        }
                    }
                } catch (Exception $e) {
                    error_log($e);
                }
                return $mailLists;
            }

            private function getMailPoetMailingLists()
            {
                if(!class_exists('WYSIJA')) return false;

                $mailLists = array();

                $model_list = WYSIJA::get('list','model');

                $lists = $model_list->get(array('name','list_id'),array('is_enabled'=>1));

                foreach ($lists as $listItem) {
                    array_push($mailLists, array($listItem['list_id'], $this->sanitizeName($listItem['name'])));
                }

                return $mailLists;
            }

            private function getMailRelayMailingLists(){

                if(!$this -> validateSettings(array('mailrelay-api-host','mailrelay-api-key'))) return false;

                $mailrelay = new LAMailRelayWrapper($this->ml_settings['mailrelay-api-host'], $this->ml_settings['mailrelay-api-key']);

                $mailLists = array();

                $resp = $mailrelay -> getMailingLists();

                if($resp && $resp['status'] == 1) {
                    foreach ($resp['data'] as $listItem) {
                        array_push($mailLists, array($listItem['id'], $this->sanitizeName($listItem['name'])));
                    }
                }

                return $mailLists;
            }

            private function getEGoiMailingLists(){
                if(!$this -> validateSettings(array('e-goi-api-key'))) return false;
                // we need to correctly handle the case when there're no lists
                $api = EgoiApiFactory::getApi(Protocol::Rest);
                $args = array('apikey' => $this->ml_settings['e-goi-api-key'], 'plugin_key' => LAGatsbyMailingListWrapper::EGOI_PLUGIN_KEY);

                $resp = $api->getLists($args);

                $mailLists = array();

                if(is_array($resp)) {
                    foreach ($resp as $listItem) {
                        array_push($mailLists, array($listItem['listnum'], $this->sanitizeName($listItem['title'])));
                    }
                }

                return $mailLists;
            }

            private function getMadMimiMailingLists(){
                if(!$this -> validateSettings(array('madmimi-api-username', 'madmimi-api-key'))) return false;

                $mimi = new MadMimi($this->ml_settings['madmimi-api-username'], $this->ml_settings['madmimi-api-key'], false);


                $mailLists = array();

                $lists_str = $mimi->Lists(); // well, this shit was modified to call json endpoint instead of xml

                $lists = json_decode($lists_str, true);

                if(is_array($lists)) {
                    foreach ($lists as $listItem) {
                        array_push($mailLists, array($listItem['id'], $this->sanitizeName($listItem['name'])));
                    }
                }

                return $mailLists;
            }

            private function getCampaignMonitorMailingLists(){
                if(!$this -> validateSettings(array('campaignmonitor-api-client-id', 'campaignmonitor-api-key'))) return false;
                // we need to correctly handle the case when there're no lists

                $mailLists = array();

                $auth = array('api_key' => $this->ml_settings['campaignmonitor-api-key']);

                $wrap = new CS_REST_Clients($this->ml_settings['campaignmonitor-api-client-id'], $auth);

                $result = $wrap->get_lists();

                if(is_array($result -> response)) {
                    foreach ($result -> response as $listItem) {
                        array_push($mailLists, array($listItem -> ListID, $this->sanitizeName($listItem -> Name)));
                    }
                }

                return $mailLists;
            }

            private function getFreshmailMailingLists(){
                if(!$this -> validateSettings(array('freshmail-api-key', 'freshmail-api-secret'))) return false;
                $mailLists = array();

                $fm = new FmRestApi();
                $fm->setApiSecret($this->ml_settings['freshmail-api-secret']);
                $fm->setApiKey($this->ml_settings['freshmail-api-key']);

                $result = $fm->doRequest('subscribers_list/lists');

                if($result['status'] == 'OK' && is_array($result['lists'])) {
                    foreach ($result['lists'] as $listItem) {
                        array_push($mailLists, array($listItem['subscriberListHash'], $this->sanitizeName($listItem['name'])));
                    }
                }

                return $mailLists;
            }

            public function addSubscriber($email, $first_name = NULL, $last_name = NULL, $phone = NULL, $city = NULL)
            {
                $response = array();

                if (isset($email) && filter_var($email, FILTER_VALIDATE_EMAIL)) {
                    try {
                        switch ($this->ml_type) {
                            case 'my-email':
                                # send an email
                                $response = $this->addEmailSubscriber($email, $first_name, $last_name, $phone, $city);
                                break;
                            case 'mailchimp':
                                # push to mailchimp with its api
                                $response = $this->addMailchimpSubscriber($email, $first_name, $last_name);
                                break;
                            case 'getresponse':
                                # push to getresponse with its api
                                $response = $this->addGetResponseSubscriber($email, $first_name, $last_name);
                                break;
                            case 'constantcontact':
                                # push to constant contact with its api
                                $response = $this->addConstantContactSubscriber($email, $first_name, $last_name, $phone);
                                break;
                            case 'icontact':
                                # push to iContact with its api
                                $response = $this->addIContactSubscriber($email, $first_name, $last_name, $phone, $city); # can take more than that
                                break;
                            case 'benchmarkmail':
                                # push to BenchmarkMail with its api
                                $response = $this->addBenchmarkMailSubscriber($email, $first_name, $last_name); # can only take email/first/last
                                break;
                            case 'mymail':
                                # push to MyMail with its, well, they call it "api"
                                $response = $this->addMyMailSubscriber($email, $first_name, $last_name);
                                break;
                            case 'aweber':
                                # push to AWeber with its api
                                $response = $this->addAWeberSubscriber($email, $first_name, $last_name, $phone, $city);
                                break;
                            case 'mailpoet':
                                $response = $this->addMailPoetSubscriber($email, $first_name, $last_name);
                                break;
                            case 'mailrelay':
                                $response = $this->addMailRelaySubscriber($email, $first_name, $last_name);
                                break;
                            case 'e-goi':
                                $response = $this->addEGoiSubscriber($email, $first_name, $last_name, $phone);
                                break;
                            case 'madmimi':
                                $response = $this->addMadMimiSubscriber($email, $first_name, $last_name, $phone, $city);
                                break;
                            case 'campaignmonitor':
                                $response = $this->addCampaignMonitorSubscriber($email, $first_name, $last_name, $phone, $city);
                                break;
                            case 'freshmail':
                                $response = $this->addFreshmailSubscriber($email, $first_name, $last_name, $phone, $city);
                                break;
                            default:
                                $response = array();
                                break;
                        }
                    }catch(Exception $e){
                        $response['error'] = 'Exception during the call.';
                        print_r("<!-- NINJA ERROR \n\n\n".var_export($e, true)."\n\n\n-->");
                    }
                } else {
                    $response['error'] = 'Invalid email address.';
                }

                return $response;
            }

            public function removeSubscriber($email)
            {


            }

	        private function addEmailSubscriber($email, $first_name, $last_name, $phone, $city)
	        {
		        $headers = array();
		        $headers[] = 'From: ' . get_bloginfo('name') . ' <' . str_replace(array("\r", "\n", "\n", "\t", ",", ";"), '', get_bloginfo('admin_email')) . ">\r\n";

		        return wp_mail($this->ml_settings['my-email'], 'NKS Form Submission',
			        'Signee email: ' . $email .  "\n" .
			        (isset($first_name) ? 'Signee name: ' . $first_name . "\n" : '') .
			        (isset($last_name) ? 'Signee lastname: ' . $last_name . "\n" : '') .
			        (isset($phone) ? 'Signee phone: ' . $phone . "\n" : '') .
			        (isset($city) ? 'Signee city: ' . $city . "\n" : ''),
			        $headers
		        );
	        }

            private function addMailchimpSubscriber($email, $first_name = NULL, $last_name = NULL)
            {
                if(!$this -> validateSettings(array('mailchimp-api-key', 'mailchimp-list-id'))) return false;
                $mc = new LA_Mailchimp($this->ml_settings['mailchimp-api-key']);

                /*try {
                    $res = $mc->lists->subscribe($this->ml_settings['mailchimp-list-id'], array('email' => $email, 'double_optin' => false, 'send_welcome' => true), array('FNAME' => $first_name, 'LNAME' => $last_name));
                    #subscribe($id, $email, $merge_vars=null, $email_type='html', $double_optin=true, $update_existing=false, $replace_interests=true, $send_welcome=false);
                    $this->ml_last_error = '';
                } catch (LA_Mailchimp_Error $e) { #Mailchimp_List_AlreadySubscribed
                    $res = false;
                    $this->ml_last_error = LAGatsbyMailingListWrapper::ALREADY_SUBSCRIBED_CODE;#$e->message;
                }*/

                try {
                    $data = array(
                        'email_address' => $email,
                        'status'        => 'subscribed'
                    );

                    if ($first_name || $last_name) {
                        $data['merge_fields'] = array();
                        if ($first_name) {
                            $data['merge_fields']['FNAME'] = $first_name;
                        }
                        if ($last_name) {
                            $data['merge_fields']['LNAME'] = $last_name;
                        }
                    }
                    $res = $mc->post("lists/" . $this->ml_settings['mailchimp-list-id'] . "/members", $data);
                    $ml_last_error = '';
                } catch (Exception $e) {
                    $res = array();
                    $res['error'] = 'Subscription error';
                }
                return $res;
            }

            private function addGetResponseSubscriber($email, $first_name, $last_name)
            {
                if(!$this -> validateSettings(array('getresponse-api-key', 'getresponse-list-id'))) return false;
                $client = new jsonRPCClient('http://api2.getresponse.com');

                try {
                    $client->add_contact(
                        $this->ml_settings['getresponse-api-key'],
                        array(
                            'campaign' => $this->ml_settings['getresponse-list-id'],
                            'name' => $first_name.' '.$last_name,
                            'email' => $email
                        )
                    );
                    $res = true;
                    $this->ml_last_error = '';
                } catch (Exception $e) {
                    $res = false;
                    if(strpos($e->getMessage(), 'Contact already') !== false){
                        $this->ml_last_error = __('You are already subscribed', 'nk-sub');
                    }else{
                        $this->ml_last_error = LAGatsbyMailingListWrapper::ALREADY_SUBSCRIBED_CODE;#  $e->getMessage();
                    }
                }

                return $res;
            }

            private function addConstantContactSubscriber($email, $first_name, $last_name, $phone)
            {
                if(!$this -> validateSettings(array('constantcontact-username', 'constantcontact-password', 'constantcontact-list-id'))) return false;

                $client = new cc($this->ml_settings['constantcontact-username'], $this->ml_settings['constantcontact-password']);
                try {
                    $client->create_contact($email, array($this->ml_settings['constantcontact-list-id']), array(
                        'first_name' => $first_name,
                        'last_name' => $last_name,
                        'cell_phone' => $phone
                    ));
                    $res = true;
                    $this->ml_last_error = '';
                } catch (Exception $e) {
                    $res = false;
                    $this->ml_last_error = LAGatsbyMailingListWrapper::ALREADY_SUBSCRIBED_CODE;#= $e->message;
                }

                return $res;
            }

            private function addIContactSubscriber($email, $first_name, $last_name, $phone, $city)
            {
                if(!$this -> validateSettings(array('icontact-api-app-id', 'icontact-api-password', 'icontact-api-username', 'icontact-list-id'))) return false;

                $oiContact = iContactApi::getInstance()->setConfig(array(
                    'appId' => $this->ml_settings['icontact-api-app-id'],
                    'apiPassword' => $this->ml_settings['icontact-api-password'],
                    'apiUsername' => $this->ml_settings['icontact-api-username']
                ));
                try {
                    $contact = $oiContact->addContact($email, 'normal', null, $first_name, $last_name, null, null, null, $city, null, null, $phone);
                    $oiContact->subscribeContactToList($contact->contactId, $this->ml_settings['icontact-list-id'], 'normal');
                    $res = true;
                    $this->ml_last_error = '';
                } catch (Exception $oException) {
                    $res = false;
                    $this->ml_last_error = LAGatsbyMailingListWrapper::ALREADY_SUBSCRIBED_CODE;#$oiContact->getErrors();
                }
                return $res;
            }

            private function addBenchmarkMailSubscriber($email, $first_name, $last_name)
            {
                if(!$this -> validateSettings(array('benchmarkmail-username', 'benchmarkmail-password', 'benchmarkmail-list-id'))) return false;

                $bmApi = new BMEAPI($this->ml_settings['benchmarkmail-username'], $this->ml_settings['benchmarkmail-password'], LAGatsbyMailingListWrapper::BENCHMARKMAIL_API_URL);

                $resp = $bmApi->listAddContacts($this->ml_settings['benchmarkmail-list-id'], array(array('email' => $email, 'firstname' => $first_name, 'lastname' => $last_name)));
                if($resp == 1){
                    $res = true;
                    $this->ml_last_error = '';
                }else{
                    $res = false;
                    $this->ml_last_error = LAGatsbyMailingListWrapper::ALREADY_SUBSCRIBED_CODE;
                }

                return $res;
            }

            private function addAweberSubscriber($email, $first_name, $last_name, $phone, $city) {
                if(!$this -> validateSettings(array('aweber-consumer-key','aweber-consumer-secret','aweber-access-key', 'aweber-access-secret', 'aweber-list-id'))) return false;

                $consumerKey = $this->ml_settings['aweber-consumer-key'];
                $consumerSecret = $this->ml_settings['aweber-consumer-secret'];
                $accessKey = $this->ml_settings['aweber-access-key'];
                $accessSecret = $this->ml_settings['aweber-access-secret'];
                try {
                    $aweber = new AWeberAPI($consumerKey, $consumerSecret);

                    $account = $aweber->getAccount($accessKey, $accessSecret);

                    $listURL = "{$account->url}/lists/{$this->ml_settings['aweber-list-id']}";
                    $list = $account->loadFromUrl($listURL);

                    # create a subscriber
                    $params = array(
                        'email' => $email,
                        'ip_address' => $this->getUserIp(),
                        'name' => implode( array_filter( array($first_name, $last_name) ), ' ' )
                    );

//                    $custom_fields = array();
//
//                    $list_custom_fields = $list->custom_fields;
//
//                    if(!is_null($phone)){
//                        try {
//                            $list_custom_fields->create(array('name' => 'phone'));
//                        }catch(AWeberAPIException $e) {
//                            // skip phone
//                        }
//                        $custom_fields['phone'] = $phone;
//                    }
//                    if(!is_null($city)){
//                        try {
//                            $list_custom_fields->create(array('name' => 'city'));
//                        }catch(AWeberAPIException $e) {
//                            // skip city
//                        }
//                        $custom_fields['city'] = $city;
//                    }
//
//                    if(count($custom_fields) > 0) $params['custom_fields'] = $custom_fields;

                    $subscribers = $list->subscribers;
                    $new_subscriber = $subscribers->create($params);
                    $this->ml_last_error = '';
                } catch(AWeberAPIException $e){
                    $this->ml_last_error = LAGatsbyMailingListWrapper::ALREADY_SUBSCRIBED_CODE; #preg_replace('/\w*\:\s*/', '', $e->message);
                }

                if(isset($new_subscriber)) {
                    $res = true;
                }else{
                    $res = false;
                }
                return $res;
            }

            private function addMyMailSubscriber($email, $first_name, $last_name)
            {
                if(!function_exists('mymail_subscribe')) return false;

                $res = mymail_subscribe( $email, array(
                    'firstname' => $first_name,
                    'lastname' => $last_name,
                ), array($this->ml_settings['mymail-list-id']) );

                return $res;
            }

            private function addMailPoetSubscriber($email, $first_name, $last_name)
            {
                if(!$this -> validateSettings(array('mailpoet-list-id'))) return false;
                if(!class_exists('WYSIJA')) return false;

                $data_subscriber = array(
                    'user' => array(
                                    'email' => $email,
                                    'firstname' => $first_name,
                                    'lastname' => $last_name
                                   ),
                    'user_list' => array('list_ids' => array($this->ml_settings['mailpoet-list-id']))
                );

                $helper_user = WYSIJA::get('user','helper');
                $res = $helper_user->addSubscriber($data_subscriber);

                return $res;
            }

            private function addMailRelaySubscriber($email, $first_name, $last_name) {
                if(!$this -> validateSettings(array('mailrelay-list-id'))) return false;

                $mailrelay = new LAMailRelayWrapper($this->ml_settings['mailrelay-api-host'], $this->ml_settings['mailrelay-api-key']);

                $name = join(' ', array($first_name, $last_name));

                $res = $mailrelay -> addSubscriber($this->ml_settings['mailrelay-list-id'], $email, $name);

                if($res['status'] == 1) {
                    $this->ml_last_error = '';
                    $result = true;
                }else{
                    $this->ml_last_error = __('You are already subscribed', 'nk-sub');
                    $result = false;
                }
                return $result;
            }

            private function addEGoiSubscriber($email, $first_name, $last_name, $phone) {
                if(!$this -> validateSettings(array('e-goi-api-key', 'e-goi-list-id'))) return false;

                $api = EgoiApiFactory::getApi(Protocol::XmlRpc);

                # status => 0/1/2/4 - Contact status (0 = unconfirmed; 1 = active; 2 = removed; 4 = inactive). Default status (or if you don't set this variable) = 0 (unconfirmed)
                $arguments = array(
                    'apikey' => $this->ml_settings['e-goi-api-key'],
                    'plugin_key' => LAGatsbyMailingListWrapper::EGOI_PLUGIN_KEY,
                    'listID' => $this->ml_settings['e-goi-list-id'],
                    'status' => 1,
                    'email' => $email,
                    'first_name' => $first_name,
                    'last_name' => $last_name
                );

                if(!is_null($phone) && $phone != ''){
                    $arguments['telephone'] = $phone;
                    $arguments['validate_phone'] = 0; // disable validation
                }

                $response = $api->addSubscriber($arguments);

                if(!is_null($response['UID']) && is_null($response['ERROR'])) {
                    $this->ml_last_error = '';
                    $result = true;
                }else{
                    $this->ml_last_error = $response['ERROR'];
                    $result = false;
                }

                return $result;
            }

            private function addMadMimiSubscriber($email, $first_name, $last_name, $phone, $city) {
                if(!$this -> validateSettings(array('madmimi-api-username', 'madmimi-api-key', 'madmimi-list-id'))) return false;

                $mimi = new MadMimi($this->ml_settings['madmimi-api-username'], $this->ml_settings['madmimi-api-key']);

                $arguments = array(
                    'email' => $email,
                    'firstName' => $first_name,
                    'lastName' => $last_name,
                    'add_list' => $this->ml_settings['madmimi-list-id']
                );

                if(!is_null($phone) && $phone != ''){ $arguments['Phone'] = $phone; }
                if(!is_null($city) && $city != ''){ $arguments['City'] = $city; }

                $response = $mimi->AddUser($arguments);

                if($response != '') {
                    $this->ml_last_error = '';
                    $result = true;
                }else{
                    $this->ml_last_error = 'Something went wrong.';
                    $result = false;
                }

                return $result;
            }

            private function addCampaignMonitorSubscriber($email, $first_name, $last_name, $phone, $city) {
                if(!$this -> validateSettings(array('campaignmonitor-api-client-id', 'campaignmonitor-api-key', 'campaignmonitor-list-id'))) return false;

                $auth = array('api_key' => $this->ml_settings['campaignmonitor-api-key']);

                $wrap = new CS_REST_Subscribers($this->ml_settings['campaignmonitor-list-id'], $auth);

                $subscriber_data = array(
                    'EmailAddress' => $email,
                    'Name' => join(' ', array($first_name, $last_name)),
                    'CustomFields' => array(),
                    'Resubscribe' => true
                );

                if(!is_null($phone) && $phone != '') $subscriber_data['CustomFields'][] = array('Key' => 'Phone', 'Value' => $phone);
                if(!is_null($city) && $city != '') $subscriber_data['CustomFields'][] = array('Key' => 'City',  'Value' => $city);

                $response = $wrap->add($subscriber_data);

                if($response->was_successful()) {
                    $this->ml_last_error = '';
                    $result = true;
                } else {
                    $this->ml_last_error = $response->response->Code.':'.$response->response->Message;
                    $result = false;
                }

                return $result;
            }

            private function addFreshmailSubscriber($email, $first_name, $last_name, $phone, $city) {
                if(!$this -> validateSettings(array('freshmail-api-key', 'freshmail-api-secret', 'freshmail-list-id'))) return false;

                $fm = new FmRestApi();
                $fm->setApiSecret($this->ml_settings['freshmail-api-secret']);
                $fm->setApiKey($this->ml_settings['freshmail-api-key']);


                $subscriber_data = array(
                    'email' => $email,
                    'list' => $this->ml_settings['freshmail-list-id'],
                    'custom_fields' => array()
                );

                $name = join(' ', array($first_name, $last_name));

                if(!is_null($name) && $name != '') $subscriber_data['custom_fields']['name'] = $name;
                if(!is_null($phone) && $phone != '') $subscriber_data['custom_fields']['phone'] = $phone;
                if(!is_null($city) && $city != '') $subscriber_data['custom_fields']['city'] = $city;

                try {
                    $response = $fm->doRequest('subscriber/add', $subscriber_data);

                    if($response['status'] == 'OK'){
                        $this->ml_last_error = '';
                        $result = true;
                    }else{
                        $result = false;
                        $this->ml_last_error = var_export($response);
                    }
                } catch (Exception $e) {
                    if($e->getCode() == '1303'){
                        try { $fm->doRequest('subscribers_list/addField', array(
                                                'hash' => $this->ml_settings['freshmail-list-id'],
                                                'name' => 'name',
                                                'tag' => 'name',
                                                'type' => 0
                                            )); } catch(Exception $e1){}

                        try { $fm->doRequest('subscribers_list/addField', array(
                                                'hash' => $this->ml_settings['freshmail-list-id'],
                                                'name' => 'phone',
                                                'tag' => 'phone',
                                                'type' => 0
                                            )); } catch(Exception $e1){}

                        try { $fm->doRequest('subscribers_list/addField', array(
                                                'hash' => $this->ml_settings['freshmail-list-id'],
                                                'name' => 'city',
                                                'tag' => 'city',
                                                'type' => 0
                                            )); } catch(Exception $e1){}

                        return $this -> addFreshmailSubscriber($email, $first_name, $last_name, $phone, $city);
                    }
                    $this->ml_last_error = $e->getCode().':'.$e->getMessage();
                    $result = false;
                }

                return $result;
            }

            private function validateSettings($required_keys){
                $valid = true;
                foreach($required_keys as $key){
                    if(is_null($this->ml_settings[$key]) || $this->ml_settings[$key] == '') $valid = false;
                }
                return $valid;
            }

            private function getUserIp() {
                if ( ! empty( $_SERVER['HTTP_CLIENT_IP'] ) ) {
                    //check ip from share internet
                    $ip = $_SERVER['HTTP_CLIENT_IP'];
                } elseif ( ! empty( $_SERVER['HTTP_X_FORWARDED_FOR'] ) ) {
                    //to check ip is pass from proxy
                    $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
                } else {
                    $ip = $_SERVER['REMOTE_ADDR'];
                }
                return apply_filters( 'wpb_get_ip', $ip );
            }

            private function sanitizeName($str) {
                return str_replace("'", '`', str_replace('"', '`', $str));
            }

        }
    }
?>
