<?php
/*
Plugin Name: Ninja Kick: Subscription
Plugin URI: sf.looks-awesome.com
Description: Push/Sliding Subscription Form on every page of your site.
Version: 1.5.6
Author: Looks Awesome
Author URI: http://looks-awesome.com
License: Commercial License
Text Domain: nk-sub
Domain Path: /lang
*/

global $nksub_options;
global $wpdb;
global $nksub_db_version;
$nksub_db_version = '1.1';

require_once(plugin_dir_path( __FILE__ ).'include/nksub_ml_settings_proxy.php');

add_action('plugins_loaded', 'mailing_includes', 9);
function mailing_includes(){
	require_once(plugin_dir_path( __FILE__ ).'include/nksub_mailing_list_wrapper.php');
} // fruitfulcode/Maintenance-PRO workaround to conditionally load mailing deps after other plugins load them unconditionally

register_activation_hook( __FILE__, 'nksub_install' );

function nksub_install() {
	global $wpdb;
	global $nksub_db_version;

	$table_name = $wpdb->prefix . 'nksubcounters';
	$subscribers_table_name = $wpdb->prefix . 'nksubsubscribers';

	/*
	 * We'll set the default character set and collation for this table.
	 * If we don't do this, some characters could end up being converted
	 * to just ?'s when saved in our table.
	 */
	$charset_collate = '';

	if ( ! empty( $wpdb->charset ) ) {
		$charset_collate = "DEFAULT CHARACTER SET {$wpdb->charset}";
	}

	if ( ! empty( $wpdb->collate ) ) {
		$charset_collate .= " COLLATE {$wpdb->collate}";
	}

	$sql = "CREATE TABLE $table_name (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
		form_id bigint(20) unsigned NOT NULL default '1',
		views bigint(20) unsigned NOT NULL default '0',
		submits bigint(20) unsigned NOT NULL default '0',
		UNIQUE KEY id (id)
	) $charset_collate;

	CREATE TABLE $subscribers_table_name (
		id mediumint(9) NOT NULL AUTO_INCREMENT,
		time datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
		form_id bigint(20) unsigned NOT NULL default '1',
		provider_used VARCHAR(64) default '',
		provider_response_positive TINYINT(1) NOT NULL default '0',
		first_name VARCHAR(64) default '',
		last_name VARCHAR(64) default '',
		email VARCHAR(128) default '',
		additional_fields_hash TEXT default '',
		UNIQUE KEY id (id)
	) $charset_collate;";

	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	dbDelta( $sql  );

	add_option( 'nksub_db_version', $nksub_db_version );
}

load_plugin_textdomain('nk-sub', false, basename( dirname( __FILE__ ) ) . '/lang' );

if (!defined('NKSUB_VERSION_KEY')) {
	define('NKSUB_VERSION_KEY', 'nksub_version');
}

if (!defined('NKSUB_VERSION_NUM')) {
	define('NKSUB_VERSION_NUM', '2.1.6');
}

add_option(NKSUB_VERSION_KEY, NKSUB_VERSION_NUM);

add_action('wp_enqueue_scripts', 'nksub_scripts');

add_action( 'wp_ajax_nksub_send_message', 'nksub_send_message' );
add_action( 'wp_ajax_nopriv_nksub_send_message', 'nksub_send_message' );

add_action("wp_ajax_nksub_count", "nksub_count");
add_action("wp_ajax_nopriv_nksub_count", "nksub_count");

add_action( 'wp_ajax_nksub_get_subscribers_csv', 'nksub_get_subscribers_csv' );

add_action( 'admin_menu', 'nksub_menu' );

function nksub_menu() {
	add_options_page( 'NK: Subscription', '<span style="display: inline-block;border-left:3px solid #5bcae9; padding-left:3px;position: relative;left: -6px;">NK: Subscription</span>', 'manage_options', 'nk-sub-options', 'nk_sub_admin_page' );
}

//include_once(dirname(__FILE__) . '/settings.php');

add_action( 'admin_init', 'nksub_register_settings_page' );

function nksub_register_settings_page () {
	register_setting( 'nksub_options', 'nksub_options', 'nksub_options_validate' );
}

function nksub_options_validate($plugin_options) {

	for ($i = 1; $i <= $plugin_options['nksub_forms']; $i++) {

		if (isset($plugin_options['nksub_form_status_' . $i]) && $plugin_options['nksub_form_status_' . $i] == 'deleted') {

			// TODO make for loop
			unset($plugin_options['nksub_is_default_' . $i]);
			unset($plugin_options['nksub_theme_' . $i]);
			unset($plugin_options['nksub_base_color_' . $i]);
			unset($plugin_options['nksub_bg_color_' . $i]);
			unset($plugin_options['nksub_image_bg_' . $i]);
			unset($plugin_options['nksub_custom_bg_' . $i]);
			unset($plugin_options['nksub_show_lining_' . $i]);
			unset($plugin_options['nksub_show_icons_' . $i]);
			unset($plugin_options['nksub_lining_bg_' . $i]);
			unset($plugin_options['nksub_gamma_' . $i]);
			unset($plugin_options['nksub_pic_' . $i]);
			unset($plugin_options['nksub_heading_' . $i]);
			unset($plugin_options['nksub_subheading_' . $i]);
			unset($plugin_options['nksub_cta_' . $i]);
			unset($plugin_options['nksub_notice_' . $i]);
			unset($plugin_options['nksub_success_message_' . $i]);
			unset($plugin_options['nksub_show_social_' . $i]);
			unset($plugin_options['nksub_facebook_' . $i]);
			unset($plugin_options['nksub_twitter_' . $i]);
			unset($plugin_options['nksub_pinterest_' . $i]);
			unset($plugin_options['nksub_instagram_' . $i]);
			unset($plugin_options['nksub_linkedin_' . $i]);
			unset($plugin_options['nksub_gplus_' . $i]);
			unset($plugin_options['nksub_rss_' . $i]);
			unset($plugin_options['nksub_form_show_' . $i]);
			unset($plugin_options['nksub_form_scroll_' . $i]);
			unset($plugin_options['nksub_form_interval_' . $i]);
			unset($plugin_options['nksub_form_locker_' . $i]);
			unset($plugin_options['nksub_form_redirect_' . $i]);
			unset($plugin_options['nksub_sel_after_' . $i]);
			unset($plugin_options['nksub_sel_before_' . $i]);
			unset($plugin_options['nksub_sel_percent_' . $i]);
			unset($plugin_options['nksub_form_days_' . $i]);
			unset($plugin_options['nksub_form_timeout_' . $i]);


//			  browser()->log($plugin_options);
//			  die();

		} else if (isset($_FILES['nksub_pic_'.$i]) && ($_FILES['nksub_pic_'.$i]['size'] > 0)) {
			// Get the type of the uploaded file. This is returned as "type/extension"
			$arr_file_type = wp_check_filetype(basename($_FILES['nksub_pic_'.$i]['name']));
			$uploaded_file_type = $arr_file_type['type'];

			// Set an array containing a list of acceptable formats
			$allowed_file_types = array('image/jpg', 'image/jpeg', 'image/gif', 'image/png');

			// If the uploaded file is the right format
			if (in_array($uploaded_file_type, $allowed_file_types)) {

				// Options array for the wp_handle_upload function. 'test_upload' => false
				$upload_overrides = array('test_form' => false);

				add_filter('wp_handle_upload_prefilter', 'nksub_upload_filter' );

				//delete previous
				//if (isset($plugin_options['nksub_pic'])) unlink($plugin_options['nksub_pic']);

				// Handle the upload using WP's wp_handle_upload function. Takes the posted file and an options array
				$uploaded_file = wp_handle_upload($_FILES['nksub_pic_'.$i], $upload_overrides);

				// If the wp_handle_upload call returned a local path for the image
				if (isset($uploaded_file['file'])) {
					// The wp_insert_attachment function needs the literal system path, which was passed back from wp_handle_upload
					$file_name_and_location = $uploaded_file['file'];
					$wp_upload_dir = wp_upload_dir();
					$plugin_options['nksub_pic_'.$i] = $wp_upload_dir['url'] . '/' . basename ($file_name_and_location);
				} else { // wp_handle_upload returned some kind of error. the return does contain error details, so you can use it here if you want.
					$upload_feedback = 'There was a problem with your upload.';
				}

			} else { // wrong file type
				$upload_feedback = 'Please upload only image files (jpg, gif or png).';
			}

		}

		if (isset($_FILES['nksub_custom_bg_' . $i]) && ($_FILES['nksub_custom_bg_' . $i]['size'] > 0)) {

			// Get the type of the uploaded file. This is returned as "type/extension"
			$arr_file_type = wp_check_filetype(basename($_FILES['nksub_custom_bg_' . $i]['name']));
			$uploaded_file_type = $arr_file_type['type'];

			// Set an array containing a list of acceptable formats
			$allowed_file_types = array('image/jpg', 'image/jpeg', 'image/gif', 'image/png');

			// If the uploaded file is the right format
			if (in_array($uploaded_file_type, $allowed_file_types)) {

				// Options array for the wp_handle_upload function. 'test_upload' => false
				$upload_overrides = array('test_form' => false);


				//delete previous
				//if (isset($plugin_options['nksub_custom_bg'])) unlink($plugin_options['nksub_custom_bg']);

				$uploaded_file = wp_handle_upload($_FILES['nksub_custom_bg_' . $i], $upload_overrides);

				// If the wp_handle_upload call returned a local path for the image
				if (isset($uploaded_file['file'])) {
					// The wp_insert_attachment function needs the literal system path, which was passed back from wp_handle_upload
					$file_name_and_location = $uploaded_file['file'];
					$wp_upload_dir = wp_upload_dir();
					$plugin_options['nksub_custom_bg_' . $i] = $wp_upload_dir['url'] . '/' . basename($file_name_and_location);
				} else { // wp_handle_upload returned some kind of error. the return does contain error details, so you can use it here if you want.
					$upload_feedback = 'There was a problem with your upload.';
				}

			} else { // wrong file type
				$upload_feedback = 'Please upload only image files (jpg, gif or png).';
			}

		} else { // No file was passed
			$upload_feedback = false;
		}
	}


	return $plugin_options;
}

/**
 * Settings page in the WP Admin
 */
function nk_sub_admin_page() {
	if ( !current_user_can( 'manage_options' ) )  {
		wp_die( __( 'You do not have sufficient permissions to access this page.', 'nk-sub' ) );
	}

	include_once(dirname(__FILE__) . '/settings.php');
	nksub_register_settings();

	wp_enqueue_script("jquery");
	wp_enqueue_script( 'nksub_tinycolor', plugins_url('/js/tinycolor.js', __FILE__) );
	wp_enqueue_script( 'nksub_colorpickersliders', plugins_url('/js/jquery.colorpickersliders.js', __FILE__) );
	wp_enqueue_script( 'nksub_admin', plugins_url('/js/admin.js', __FILE__) );

	//wp_register_style('open-sans-font', 'http://fonts.googleapis.com/css?family=Open+Sans:400,300' );
	wp_register_style('open-sans-font', '//fonts.googleapis.com/css?family=Open+Sans:300normal,400normal,400italic,600normal,600italic&subset=all' );

	wp_enqueue_style( 'open-sans-font' );
  wp_register_style('colorpickersliders-ui-css', plugins_url('/css/jquery.colorpickersliders.css', __FILE__));
	wp_enqueue_style( 'colorpickersliders-ui-css' );

  wp_register_style('nksub-admin-css', plugins_url('/css/admin.css', __FILE__));
	wp_enqueue_style( 'nksub-admin-css' );

	include_once(dirname(__FILE__) . '/options-page.php');
}


add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'nksub_plugin_action_links', 10, 1);

function nksub_plugin_action_links($links) {
	$settings_page = add_query_arg(array('page' => 'nk-sub-options'), admin_url('options-general.php'));
	$settings_link = '<a href="'.esc_url($settings_page).'">'.__('Settings', 'nk-sub' ).'</a>';
	array_unshift($links, $settings_link);
	return $links;
}

add_action('wp_head', 'nksub_dynamic_style');
add_action('wp_head', 'nksub_main_html', 1000001); // Browser will insert it right after BODY tag

function nksub_dynamic_style () {
	global $nksub_show;
	$options = nksub_get_options();
	if ($options['nksub_test_mode'] === 'yes' && !current_user_can( 'manage_options' ) ) return;

	if (isset($nksub_show) && $nksub_show) {
		include_once(dirname(__FILE__) . '/nk-sub-dynamic-styles.php');
	}
}

function nksub_scripts() {
	global $nksub_show;
	$options = nksub_get_options();
	$post_id = get_queried_object_id();
	if ($options['nksub_test_mode'] === 'yes' && !current_user_can( 'manage_options' ) ) return;

	$nksub_show = nksub_check_display_rule(json_decode($options['nksub_display']), nksub_is_mobile(), $post_id);
//	browser()->log($nksub_show);

	if ($nksub_show) {
	  wp_enqueue_script(
		  'nks_sub_main_js',
//		  plugins_url('/js/nk-sub.js', __FILE__),
			plugins_url('/js/nk-sub.min.js', __FILE__),
		  array('jquery')
	  );

	  $social = array();

	  $errors = array(
		  "required" => __("* Please enter %%", 'nk-sub' ),
		  "min" => __("* %% must have at least %% characters.", 'nk-sub' ),
		  "max" => __("* %% can have at most %% characters.", 'nk-sub' ),
		  "matches" => __("* %% must match %%.", 'nk-sub' ),
		  "less" => __("* %% must be less than %%", 'nk-sub' ),
		  "greater" => __("* %% must be greater than %%.", 'nk-sub' ),
		  "numeric" => __("* %% must be numeric.", 'nk-sub' ),
		  "email" => __("* %% must be a valid email address.", 'nk-sub' ),
		  "ip" => __("* %% must be a valid ip address.", 'nk-sub' ),
		  "answer" => __("* Wrong %%", 'nk-sub' )
	  );

		$js_opts = array(
			'ajaxurl' => admin_url( 'admin-ajax.php' ),
			'sidebar_type' => $options['nksub_sidebar_type'],
			'social' => $social,
			'sidebar_pos' => $options['nksub_sidebar_pos'],
			'fade_content' => $options['nksub_fade_content'],
			'label' => $options['nksub_label_style'],
			'label_top' => $options['nksub_label_top'],
			'label_vis' => $options['nksub_label_vis'],
			'label_scroll_selector' => $options['nksub_label_vis_selector'],
			'label_mouseover' => $options['nksub_label_mouseover'],
			'togglers' => $options['nksub_togglers'],
			'path' => plugins_url('/img/', __FILE__),
			'send_more_text' => __( "Send more", 'nk-sub' ),
			'try_again_text' => __( "Try again", 'nk-sub' ),
			'close_text' => __( "Close", 'nk-sub' ),
			'sending_text' => __( "Sending", 'nk-sub' ),
			'msg_fail_text' => __( "Something went wrong while sending your message", 'nk-sub' ),
			'errors' => $errors
		);

		for ($i = 1; $i <= $options['nksub_forms']; $i++) {
			if ($options['nksub_form_status_' . $i] !== 'live') continue;
			if (!empty($options['nksub_facebook_' . $i])) $social[$i]['facebook'] = $options['nksub_facebook_' . $i];
			if (!empty($options['nksub_twitter_' . $i])) $social[$i]['twitter'] = $options['nksub_twitter_' . $i];
			if (!empty($options['nksub_gplus_' . $i])) $social[$i]['gplus'] = $options['nksub_gplus_' . $i];
			if (!empty($options['nksub_instagram_' . $i])) $social[$i]['instagram'] = $options['nksub_instagram_' . $i];
			if (!empty($options['nksub_pinterest_' . $i])) $social[$i]['pinterest'] = $options['nksub_pinterest_' . $i];
			if (!empty($options['nksub_youtube_' . $i])) $social[$i]['youtube'] = $options['nksub_youtube_' . $i];
			if (!empty($options['nksub_linkedin_' . $i])) $social[$i]['linkedin'] = $options['nksub_linkedin_' . $i];
			if (!empty($options['nksub_rss_' . $i])) $social[$i]['rss'] = $options['nksub_rss_' . $i];
//			$js_opts['id'.$i]['success'] = $options['nksub_success_message_' . $i];
			$js_opts['id'.$i]['show'] = $options['nksub_form_show_' . $i];
			$js_opts['id'.$i]['timeout'] = $options['nksub_form_timeout_' . $i];
			$js_opts['id'.$i]['scroll'] = $options['nksub_form_scroll_' . $i];
			if ($options['nksub_form_show_' . $i] === 'scroll') {
				$js_opts['id'.$i]['scroll_after'] = $options['nksub_sel_after_' . $i];
				$js_opts['id'.$i]['scroll_before'] = $options['nksub_sel_before_' . $i];
				$js_opts['id'.$i]['scroll_percent'] = $options['nksub_sel_percent_' . $i];
			}
			$js_opts['id'.$i]['id'] = $i;
			$js_opts['id'.$i]['locker'] = $options['nksub_form_locker_' . $i];
			$js_opts['id'.$i]['intr'] = $options['nksub_form_interval_' . $i];
			$js_opts['id'.$i]['days'] = $options['nksub_form_days_' . $i];
			$js_opts['id'.$i]['close'] = $options['nksub_form_close_' . $i];
			$js_opts['id'.$i]['lock'] = $options['nksub_form_locker_' . $i];
			$js_opts['id'.$i]['redir'] = $options['nksub_form_redirect_' . $i];
		}

		$js_opts['social'] = $social;

		//browser()->log($js_opts['social'] );

	  wp_localize_script( 'nks_sub_main_js', 'NKSubOpts', $js_opts);

	  wp_register_style( 'nksub_styles', plugins_url('/css/nk-sub.css', __FILE__) );
	  wp_enqueue_style( 'nksub_styles' );
  }
}

function nksub_main_html() {
	global $nksub_show;
	$options = nksub_get_options();
	if ($options['nksub_test_mode'] === 'yes' && !current_user_can( 'manage_options' ) ) return;

	if (isset($nksub_show) && $nksub_show) {
		include_once(dirname(__FILE__) . '/nk-sub.php');
	}
}

function nksub_count() {

	global $wpdb;

	if ( !wp_verify_nonce( $_REQUEST['nonce'], "nksub_form_" . $_REQUEST['form'] . "_nonce")) {
		exit("No naughty business please");
	}

	$table_name = $wpdb->prefix . 'nksubcounters';

	// todo check if number
	$result = $wpdb->get_results ("SELECT views FROM {$table_name} WHERE form_id = '". $_REQUEST['form'] ."'");

	//browser()->log('db select views');
	//browser()->log($result);

	if (count ($result) > 0) {
		$row = current ($result);
		$op = $wpdb->query ("UPDATE {$table_name} SET views = ".($row->views + 1)." WHERE form_id = '". $_REQUEST['form'] ."'");
	} else {
		$op = $wpdb->query ("INSERT INTO {$table_name} (form_id, views, submits, time) VALUES (" . $_REQUEST['form'] . ", 1, 0, '" . current_time( 'mysql' ) . "' )");
	}

	if ($op) {
		echo json_encode(array('counted' => true, 'result' => $op));
	} else {
		echo json_encode(array('counted' => false, 'result' => $op));
	}

	die();
}

function nksub_send_message() {

	global $wpdb;
	$options = nksub_get_options();
	$id = $_POST['form_id'];
	$table_name = $wpdb->prefix . 'nksubcounters';
	$subscribers_table_name = $wpdb->prefix . 'nksubsubscribers';

    $email = $_POST['nksub_email_field'];
    $full_name = isset($_POST['nksub_full_name_field']) ? $_POST['nksub_full_name_field'] : '';
    $full_name_parts = preg_split("/\s+/", $full_name);
    $first_name = array_shift($full_name_parts);
    $last_name = join(' ', $full_name_parts);
	$city = isset($_POST['nksub_city_field']) ? $_POST['nksub_city_field']  : '';
	$phone = isset($_POST['nksub_phone_field']) ? $_POST['nksub_phone_field']  : '';

	if(!empty($first_name) && empty($last_name)) { // try to split name
		$parts = explode(' ', $first_name);
		if(count($parts) > 1){
			$first_name = $parts[0];
			$last_name = $parts[1];
		}
	}

  	$settings_proxy = new LAGatsbyMLSettingsProxy($options);
  	$mlm = new LAGatsbyMailingListWrapper($settings_proxy -> getType(), $settings_proxy ->getSettings());
  	$result = $mlm -> addSubscriber($email, $first_name, $last_name, $phone, $city);

	$additional_fields = array();
	if(!empty($phone)) $additional_fields['phone'] = $phone;
	if(!empty($city)) $additional_fields['city'] = $city;

	$wpdb -> insert(
		$subscribers_table_name,
		array(
			'time' => current_time('mysql', 1),
			'form_id' => $id,
			'provider_used' => $settings_proxy -> getType(),
			'provider_response_positive' => isset($result) && $result,
			'first_name' => $first_name,
			'last_name' => $last_name,
			'email' => $email,
			'additional_fields_hash' => serialize($additional_fields)
		)
	);

	if (isset($result) && $result) {

		$query = $wpdb->get_results ("SELECT submits FROM {$table_name} WHERE form_id = '". $id ."'");

		//browser()->log('db select submits for id ' . $id);
		//browser()->log($result);

		if (count ($query) > 0) {
			$row = current ($query);
			$query = $wpdb->query ("UPDATE {$table_name} SET submits = ".($row->submits + 1)." WHERE form_id = '". $id ."'");
		}

		echo json_encode(array('success' => $result, 'error_message'));


	}else{
		echo json_encode(array('success' => false, 'error_code' => $mlm -> getLastError(), 'message' => __("Message not sent. An unknown error occurred.", 'nk-sub' ), 'result' => $result));
	}

  	die();
}

function nksub_get_ip_address() {
 foreach (array('HTTP_CLIENT_IP', 'HTTP_X_FORWARDED_FOR', 'HTTP_X_FORWARDED', 'HTTP_X_CLUSTER_CLIENT_IP', 'HTTP_FORWARDED_FOR', 'HTTP_FORWARDED', 'REMOTE_ADDR') as $key) {
     if (array_key_exists($key, $_SERVER) === true) {
         foreach (explode(',', $_SERVER[$key]) as $ip) {
            if (filter_var($ip, FILTER_VALIDATE_IP) !== false) {
            return $ip;
          }
        }
     }
  }
}

function nksub_is_mobile() {
	return preg_match("/(android|avantgo|blackberry|bolt|boost|cricket|docomo|fone|hiptop|mini|mobi|palm|phone|pie|tablet|up\.browser|up\.link|webos|wos)/i", $_SERVER["HTTP_USER_AGENT"]);
}

function nksub_get_lang_id($id, $type = 'page') {
	if ( function_exists('icl_object_id') ) {
		$id = icl_object_id($id, $type, true);
	}

	return $id;
}

function nksub_check_location ($opt, $post_id) {
	if ( is_home() ) {

		//browser()->log  ( 'home' );

		$show = isset($opt->location->wp_pages->home);
		if ( !$show && $post_id ){
			$show = isset($opt->location->pages->$post_id);
		}

		// check if blog page is front page too
		if ( !$show && is_front_page() /*&& isset($opt['page-front'])*/ ) {
			//browser()->log  ( 'home front' );
			$show = isset($opt->location->wp_pages->front);
		}

	} else if ( is_front_page() ) {
		//browser()->log  ( 'front' );

		$show = isset($opt->location->wp_pages->front);
		if ( !$show && $post_id ) {
			$show = isset($opt->location->pages->$post_id);
		}
	} else if ( is_category() ) {
		//browser()->log  ( 'cat' );
		//browser()->log  ( get_query_var('cat') );
		$catid = get_query_var('cat');
		$show = isset($opt->location->cats->$catid);
	} /*else if ( is_tax() ) {
				$term = get_queried_object();
				$tax = $term->taxonomy;
				$show = isset($opt->location->cats->$tax);
				unset($term);
				unset($tax);
			} else if ( is_post_type_archive() ) {
				$type = get_post_type();
				$show = isset($opt['type-'. $type .'-archive']) ? $opt['type-'. $type .'-archive'] : false;
			}*/ else if ( is_archive() ) {
		//browser()->log  ( 'archive' );

		$show = isset($opt->location->wp_pages->archive);
	} else if ( is_single() ) {
		//browser()->log  ( 'single' );

		$type = get_post_type();
		$show = isset($opt->location->wp_pages->single);

		if ( !$show  && $type != 'page' && $type != 'post') {
			$show = isset($opt->location->cposts->$type);
		}

		if ( !$show ) {
			$cats = get_the_category();
			foreach ( $cats as $cat ) {
				if ($show) break;
				$c_id = nksub_get_lang_id($cat->cat_ID, 'category');
				$show = isset($opt->location->cats->$c_id);
				unset($c_id);
				unset($cat);
			}
		}

	} else if ( is_404() ) {
		$show = isset($opt->location->wp_pages->forbidden);
		//browser()->log  ( '404' );
		//browser()->log  ( isset($opt->location->wp_pages->forbidden));

	} else if ( is_search() ) {
		//browser()->log  ( 'search' );

		$show = isset($opt->location->wp_pages->search);
	} else if ( $post_id ) {
		//browser()->log  ( 'post id' );

		$show = isset($opt->location->pages->$post_id);
	} else {
		//browser()->log  ( 'super else' );

		$show = false;
	}

	if ( $post_id && !$show && isset($opt->location->ids) && !empty($opt->location->ids) ) {
		//browser()->log  ( 'ids' );

		$other_ids = $opt->location->ids;
		foreach ( $other_ids as $other_id ) {
			if ( $post_id == (int) $other_id ) {
				$show = true;
			}
		}
	}

	if ( !$show && defined('ICL_LANGUAGE_CODE') ) {
		// check for WPML widgets
        $lang = ICL_LANGUAGE_CODE;
        $show = isset($opt->location->langs->$lang );
    }


	if ( !isset($show) ) {
		//browser()->log  ( '!isset($show)' );
		$show = false;
	}

	return $show;
}

function nksub_check_display_rule($opt, $isMobile, $post_id) {

//	browser()->log  ( $opt );
//	browser()->log($isMobile);

	$show = nksub_check_location($opt, $post_id);

	//browser()->log  ( '>>>> inclusion' );
	//browser()->log  ( $show );

	if ($show && $opt->rule->exclude || !$show && $opt->rule->include ) {
		$show = false;
	} else  {
		$show = true;
	}

	$user_ID = is_user_logged_in();
	//browser()->log  ( '>>>> loggedin' );
	//browser()->log  ( $user_ID );
	//browser()->log  ( '>>>> checked' );
	//browser()->log  ( $show );

	if ( ( $opt->user->loggedout && $user_ID ) || ( $opt->user->loggedin && !$user_ID ) ) {
		$show = false;
	}

	if ( $opt->mobile->no && $isMobile) {
		$show = false;
	}

	return $show;
}

global $nksub_cached_opts;

function nksub_get_options()
{
	global $nksub_cached_opts;
	global $wpdb;

	if (isset($nksub_cached_opts)) return $nksub_cached_opts;

	$options = get_option('nksub_options');

	if (empty($options['nksub_forms'])) {
		$options['nksub_forms'] = 1;
	}

	if (empty($options['nksub_test_mode'])) {
		$options['nksub_test_mode'] = '';
	}

	$options['locations'] = nksub_get_locations();

	$table_name = $wpdb->prefix . 'nksubcounters';
	$options['counters'] = $wpdb->get_results ("SELECT * FROM {$table_name}");

	// FORMS

	if (!isset($options['nksub_is_default_1'])) {

		$options['nksub_is_default_1'] = (object)array(
			"location" => (object)array(
					"pages" => (object)array(),
					"cposts" => (object)array(),
					"cats" => (object)array(),
					"taxes" => (object)array(),
					"langs" => (object)array(),
					"wp_pages" => (object)array(),
					"ids" => array()
				)
		);

		foreach ($options['locations']->wp_pages as $key => $label) {
			$options['nksub_is_default_1']->location->wp_pages->$key = true;
		}

		foreach ( $options['locations']->pages as $page ) {
			$id = $page->ID;
			$options['nksub_is_default_1']->location->pages->$id = true;
		}

		if ( !empty($options['locations']->cposts) ) {
			foreach ( $options['locations']->cposts as $post_key => $custom_post ) {
				$options['nksub_is_default_1']->location->cposts->$post_key = true;
				unset($post_key);
				unset($custom_post);
			}
		}

		foreach ( $options['locations']->cats as $cat ) {
			$catid = $cat->cat_ID;
			$options['nksub_is_default_1']->location->cats->$catid = true;
			unset($cat);
		}

		if ( !empty($options['locations']->langs) ) {
			foreach ( $options['locations']->langs as $lang ) {
				$key = $lang['language_code'];
				$options['nksub_is_default_1']->location->langs->$key = true;
				unset($lang);
				unset($key);
			}
		}

		$options['nksub_is_default_1'] =  json_encode($options['nksub_is_default_1']);
	}

	for ($i = 1; $i <= $options['nksub_forms']; $i++) {

		if (!empty($options['nksub_form_status_' . $i]) && $options['nksub_form_status_' . $i] === 'deleted') {
			continue;
		}

		if (empty($options['nksub_form_status_' . $i])) {
			$options['nksub_form_status_' . $i] = 'live';
		}

		// THEME
		if (empty($options['nksub_theme_' . $i])) {
			$options['nksub_theme_' . $i] = 'flat';
		}

		if (empty($options['nksub_base_color_' . $i])) {
			$options['nksub_base_color_' . $i] = '#50E3C2';
		}

		if (empty($options['nksub_bg_color_' . $i])) {
			$options['nksub_bg_color_' . $i] = '#55534F';
		}

		if (empty($options['nksub_gamma_' . $i])) {
			$options['nksub_gamma_' . $i] = 'light';
		}

		if (empty($options['nksub_color_schema_' . $i])) {
			$options['nksub_color_schema_' . $i] = '#50E3C2,#cf4739,#cd3424,#d9593e,#c84c3f,#bb2d1f,#e96d3d,#e94e3d,#2f1420';
		}

		if (empty($options['nksub_image_bg_' . $i])) {
			$options['nksub_image_bg_' . $i] = 'none';
		}
		if (empty($options['nksub_custom_bg_' . $i])) {
			$options['nksub_custom_bg_' . $i] = '';
		}
		///

		if (empty($options['nksub_additional_fields_' . $i])) {
			$options['nksub_additional_fields_' . $i] = '{"name" : false, "company" : false, "phone" : false, "address" : false, "subject" : false}';
		}

		if (empty($options['nksub_additional_company_' . $i])) {
			$options['nksub_additional_company_' . $i] = '';
		}

		if (empty($options['nksub_additional_phone_' . $i])) {
			$options['nksub_additional_phone_' . $i] = '';
		}

		if (empty($options['nksub_additional_address_' . $i])) {
			$options['nksub_additional_address_' . $i] = '';
		}

		if (empty($options['nksub_additional_subject_' . $i])) {
			$options['nksub_additional_subject_' . $i] = '';
		}

		// DISPLAYING nksub_form_show_

		if (empty($options['nksub_form_show_' . $i])) {
			$options['nksub_form_show_' . $i] = 'pageload';
		}

		if (empty($options['nksub_form_scroll_' . $i])) {
			$options['nksub_form_scroll_' . $i] = 'start';
		}

		if (empty($options['nksub_form_interval_' . $i])) {
			$options['nksub_form_interval_' . $i] = 'cookie';
		}

		if (empty($options['nksub_form_timeout_' . $i])) {
			$options['nksub_form_timeout_' . $i] = '0';
		}

		if (empty($options['nksub_form_days_' . $i])) {
			$options['nksub_form_days_' . $i] = '1';
		}

		if (empty($options['nksub_form_locker_' . $i])) {
			$options['nksub_form_locker_' . $i] = '';
		}

		if (empty($options['nksub_form_redirect_' . $i])) {
			$options['nksub_form_redirect_' . $i] = '';
		}

		if (empty($options['nksub_sel_after_' . $i])) {
			$options['nksub_sel_after_' . $i] = '';
		}

		if (empty($options['nksub_sel_before_' . $i])) {
			$options['nksub_sel_before_' . $i] = '';
		}

		if (empty($options['nksub_sel_percent_' . $i])) {
			$options['nksub_sel_percent_' . $i] = '50';
		}

		if (empty($options['nksub_form_close_' . $i])) {
			$options['nksub_form_close_' . $i] = '';
		}

		// PROFILE

		if (!isset($options['nksub_pic_' . $i])) {
			$options['nksub_pic_' . $i] = plugins_url('/img/mail.png', __FILE__);
		}

		if (!isset($options['nksub_heading_' . $i])) {
			$options['nksub_heading_' . $i] = 'Wanna get our awesome news?';
		}

		if (!isset($options['nksub_subheading_' . $i])) {
			$options['nksub_subheading_' . $i] = "We will send you emails only several times per week. Isn't that cool?";
		}

		if (!isset($options['nksub_cta_' . $i])) {
			$options['nksub_cta_' . $i] = "Subscribe!";
		}

		if (!isset($options['nksub_notice_' . $i])) {
			$options['nksub_notice_' . $i] = "Actually we will not spam you and keep your personal data secure";
		}

		if (empty($options['nksub_show_lining_' . $i])) {
			$options['nksub_show_lining_' . $i] = '';
		}

		if (empty($options['nksub_show_icons_' . $i])) {
			$options['nksub_show_icons_' . $i] = '';
		}
		if (empty($options['nksub_lining_bg_' . $i])) {
			$options['nksub_lining_bg_' . $i] = 'rgba(0,0,0,0.9)';
		}

		// SOCIAL

		if (empty($options['nksub_facebook_' . $i])) {
			$options['nksub_facebook_' . $i] = '';
		}
		if (empty($options['nksub_twitter_' . $i])) {
			$options['nksub_twitter_' . $i] = '';
		}
		if (empty($options['nksub_pinterest_' . $i])) {
			$options['nksub_pinterest_' . $i] = '';
		}
		if (empty($options['nksub_youtube_' . $i])) {
			$options['nksub_youtube_' . $i] = '';
		}
		if (empty($options['nksub_instagram_' . $i])) {
			$options['nksub_instagram_' . $i] = '';
		}
		if (empty($options['nksub_linkedin_' . $i])) {
			$options['nksub_linkedin_' . $i] = '';
		}
		if (empty($options['nksub_gplus_' . $i])) {
			$options['nksub_gplus_' . $i] = '';
		}
		if (empty($options['nksub_rss_' . $i])) {
			$options['nksub_rss_' . $i] = '';
		}

		if (empty($options['nksub_success_message_' . $i])) {
			$options['nksub_success_message_' . $i] = __("You have been successfully subscribed to our newsletter!", 'nk-sub' );
		}

		if (empty($options['nksub_callback_' . $i])) {
			$options['nksub_callback_' . $i] = '';
		}

		if (empty($options['nksub_form_status_' . $i])) {
			$options['nksub_form_status_' . $i] = 'live';
		}

		if (empty($options['nksub_is_default_' . $i])) {
			$opts = (object)array(
				"location" => (object)array(
						"pages" => (object)array(),
						"cposts" => (object)array(),
						"cats" => (object)array(),
						"taxes" => (object)array(),
						"langs" => (object)array(),
						"wp_pages" => (object)array(),
						"ids" => array()
					)
			);
			$options['nksub_is_default_' . $i] =  json_encode($opts);
		}
	}
	// THEME



	if (empty($options['nksub_custom_css'])) {
		$options['nksub_custom_css'] = '';
	}


	// GENERAL
	if (empty($options['nksub_sidebar_type'])) {
		$options['nksub_sidebar_type'] = 'push';
	}


	if (empty($options['nksub_display'])) {
		$opts = (object)array(
			"user" => (object)array(
					"everyone" => 1,
					"loggedin" => 0,
					"loggedout" => 0
				),
			"mobile" => (object)array(
					"yes" => 1,
					"no" => 0
				),
			"rule" => (object)array(
					"include" => 0,
					"exclude" => 1
				),
			"location" => (object)array(
					"pages" => (object)array(),
					"cposts" => (object)array(),
					"cats" => (object)array(),
					"taxes" => (object)array(),
					"langs" => (object)array(),
					"wp_pages" => (object)array(),
					"ids" => array()
				)
		);
		$options['nksub_display'] =  json_encode($opts);
	}

	if (empty($options['nksub_enable_test'])) {
		$options['nksub_enable_test'] = false;
	}

	if (empty($options['nksub_use_admin_email'])) {
		$options['nksub_use_admin_email'] = false;
	}

	if (empty($options['nksub_fade_content'])) {
		$options['nksub_fade_content'] = 'light';
	}

	if (empty($options['nksub_sidebar_pos'])) {
		$options['nksub_sidebar_pos'] = 'left';
	}

	if (empty($options['nksub_label_color'])) {
		$options['nksub_label_color'] = 'rgb(80, 227, 194)';
	}

	if (empty($options['nksub_label_style'])) {
		$options['nksub_label_style'] = 'mail-1';
	}
	if (empty($options['nksub_label_size'])) {
		$options['nksub_label_size'] = '2x';
	}

	if (empty($options['nksub_label_tooltip'])) {
		$options['nksub_label_tooltip'] = 'hover';
	}

	if (empty($options['nksub_label_tooltip_text'])) {
		$options['nksub_label_tooltip_text'] = 'Subscribe';
	}

	if (empty($options['nksub_tooltip_color'])) {
		$options['nksub_tooltip_color'] = 'rgba(0, 0, 0, 0.7)';
	}

	if (empty($options['nksub_label_shape'])) {
		$options['nksub_label_shape'] = 'square';
	}

	if (empty($options['nksub_metro'])) {
		$options['nksub_metro'] = '';
	}

	if (empty($options['nksub_label_invert'])) {
		$options['nksub_label_invert'] = '';
	}

	if (empty($options['nksub_label_top'])) {
		$options['nksub_label_top'] = '50%';
	}

	if (empty($options['nksub_label_top_mob'])) {
		$options['nksub_label_top_mob'] = '100px';
	}

	if (empty($options['nksub_label_vis'])) {
		$options['nksub_label_vis'] = 'visible';
	}
	if (empty($options['nksub_label_vis_selector'])) {
		$options['nksub_label_vis_selector'] = '';
	}

	if (empty($options['nksub_label_mouseover'])) {
		$options['nksub_label_mouseover'] = '';
	}

	if (empty($options['nksub_togglers'])) {
		$options['nksub_togglers'] = '';
	}

	if (empty($options['nksub_counters'])) {
		$options['nksub_counters'] = '';
	}

	// MAILING
	if (empty($options['nksub_mail_type'])) {
		$options['nksub_mail_type'] = 'email';
	}

	if (empty($options['nksub_email'])) {
		$options['nksub_email'] = get_bloginfo('admin_email');
	}

	if (empty($options['nksub_mc_key'])) {
		$options['nksub_mc_key'] = '';
	}
	if (empty($options['nksub_mc_list_id'])) {
		$options['nksub_mc_list_id'] = '';
	}
	if (empty($options['nksub_mc_lists'])) {
		$options['nksub_mc_lists'] = '';
	}

	if (empty($options['nksub_gr_key'])) {
		$options['nksub_gr_key'] = '';
	}
	if (empty($options['nksub_gr_list_id'])) {
		$options['nksub_gr_list_id'] = '';
	}
	if (empty($options['nksub_gr_lists'])) {
		$options['nksub_gr_lists'] = '';
	}

	if (empty($options['nksub_cc_user'])) {
		$options['nksub_cc_user'] = '';
	}
	if (empty($options['nksub_cc_pass'])) {
		$options['nksub_cc_pass'] = '';
	}
	if (empty($options['nksub_cc_list_id'])) {
		$options['nksub_cc_list_id'] = '';
	}
	if (empty($options['nksub_cc_lists'])) {
		$options['nksub_cc_lists'] = '';
	}


	if (empty($options['nksub_ic_app'])) {
		$options['nksub_ic_app'] = '';
	}

	if (empty($options['nksub_ic_user'])) {
		$options['nksub_ic_user'] = '';
	}
	if (empty($options['nksub_ic_pass'])) {
		$options['nksub_ic_pass'] = '';
	}
	if (empty($options['nksub_ic_list_id'])) {
		$options['nksub_ic_list_id'] = '';
	}
	if (empty($options['nksub_ic_lists'])) {
		$options['nksub_ic_lists'] = '';
	}

	if (empty($options['nksub_bm_user'])) {
		$options['nksub_bm_user'] = '';
	}
	if (empty($options['nksub_bm_pass'])) {
		$options['nksub_bm_pass'] = '';
	}
    if (empty($options['nksub_bm_list_id'])) {
        $options['nksub_bm_list_id'] = '';
    }
    if (empty($options['nksub_bm_lists'])) {
        $options['nksub_bm_lists'] = '';
    }

    if (empty($options['nksub_mm_list_id'])) {
        $options['nksub_mm_list_id'] = '';
    }
    if (empty($options['nksub_mm_lists'])) {
        $options['nksub_mm_lists'] = '';
    }

    if (empty($options['nksub_aw_auth_code'])) {
        $options['nksub_aw_auth_code'] = '';
    }
    if (empty($options['nksub_aw_list_id'])) {
        $options['nksub_aw_list_id'] = '';
    }
    if (empty($options['nksub_aw_lists'])) {
        $options['nksub_aw_lists'] = '';
    }
	if (empty($options['nksub_aw_details_for'])) {
		$options['nksub_aw_details_for'] = '';
	}
	if (empty($options['nksub_aw_consumer_key'])) {
		$options['nksub_aw_consumer_key'] = '';
	}
	if (empty($options['nksub_aw_consumer_secret'])) {
		$options['nksub_aw_consumer_secret'] = '';
	}
	if (empty($options['nksub_aw_access_key'])) {
		$options['nksub_aw_access_key'] = '';
	}
	if (empty($options['nksub_aw_access_secret'])) {
		$options['nksub_aw_access_secret'] = '';
	}

	if (empty($options['nksub_mr_api_host'])) {
		$options['nksub_mr_api_host'] = '';
	}
	if (empty($options['nksub_mr_api_key'])) {
		$options['nksub_mr_api_key'] = '';
	}
	if (empty($options['nksub_mr_list_id'])) {
		$options['nksub_mr_list_id'] = '';
	}
	if (empty($options['nksub_mr_lists'])) {
		$options['nksub_mr_lists'] = '';
	}


	if (empty($options['nksub_eg_api_key'])) {
		$options['nksub_eg_api_key'] = '';
	}
	if (empty($options['nksub_eg_list_id'])) {
		$options['nksub_eg_list_id'] = '';
	}
	if (empty($options['nksub_eg_lists'])) {
		$options['nksub_eg_lists'] = '';
	}


	if (empty($options['nksub_mmi_api_username'])) {
		$options['nksub_mmi_api_username'] = '';
	}
	if (empty($options['nksub_mmi_api_key'])) {
		$options['nksub_mmi_api_key'] = '';
	}
	if (empty($options['nksub_mmi_list_id'])) {
		$options['nksub_mmi_list_id'] = '';
	}
	if (empty($options['nksub_mmi_lists'])) {
		$options['nksub_mmi_lists'] = '';
	}


	if (empty($options['nksub_cm_api_client_id'])) {
		$options['nksub_cm_api_client_id'] = '';
	}
	if (empty($options['nksub_cm_api_key'])) {
		$options['nksub_cm_api_key'] = '';
	}
	if (empty($options['nksub_cm_list_id'])) {
		$options['nksub_cm_list_id'] = '';
	}
	if (empty($options['nksub_cm_lists'])) {
		$options['nksub_cm_lists'] = '';
	}

	if (empty($options['nksub_fm_api_key'])) {
		$options['nksub_fm_api_key'] = '';
	}
	if (empty($options['nksub_fm_api_secret'])) {
		$options['nksub_fm_api_secret'] = '';
	}

	$nksub_cached_opts = $options;

//	browser()->log  ( $options );

//	delete_option('nksub_options');
	return $options;
}

function nksub_get_locations () {
	global $nksub_locations;

	if (isset($nksub_locations)) return $nksub_locations;

	$locations = new stdClass();

	// pages on site
	$pages = array();
	$fields = array('post_title', 'ID');

	$posts = get_posts( array(
		'post_type' => 'page',
		'post_status' => 'publish',
		'numberposts' => -1,
		'orderby' => 'title',
		'order' => 'ASC',
		'fields' => 'ids, titles'
	));

	foreach($posts as $post) {
		$newPost = new stdClass();
		foreach($fields as $field) {
			$newPost->$field = $post->$field;
		}
		$pages[] = $newPost;
	}

	$locations->pages = $pages;

	// custom post types
	$locations->cposts = get_post_types( array(
		'public' => true,
	), 'object');

	foreach ( array( 'revision', 'post', 'page', 'attachment', 'nav_menu_item' ) as $unset ) {
		unset($locations->cposts[$unset]);
	}

	foreach ( $locations->cposts as $c => $type ) {
		$post_taxes = get_object_taxonomies($c);
		foreach ( $post_taxes as $post_tax) {
			$locations->taxes[] = $post_tax;
		}
	}

	// categories
	$locations->cats = get_categories( array(
		'hide_empty'    => false,
		//'fields'        => 'id=>name', //added in 3.8
	) );

	// WPML languages
	if (function_exists('icl_get_languages') ) {
		//browser()->log('detect langs');
		$locations->langs = icl_get_languages('skip_missing=0&orderby=code');
	}

	foreach ( $locations as $key => $val ) {

		if (!empty($val)) {
			$length = count($val);
			for ($i = 0; $i <= $length; $i++) {
				if (isset($val[$i])) {
					//browser()->log  ( $val[$i] );
				}
			}
		}
	}

	$page_types = array(
		'front'     => __('Front', 'nk-sub'),
		'home'      => __('Home/Blog', 'nk-sub'),
		'archive'   => __('Archives'),
		'single'    => __('Single Post'),
		'forbidden' => '404',
		'search'    => __('Search'),
	);

	foreach ($page_types as $key => $label){
		//browser()->log  ( $key, $label );
		//$instance['page-'. $key] = isset($instance['page-'. $key]) ? $instance['page-'. $key] : false;
	}

	$locations->wp_pages = $page_types;

	$nksub_locations = $locations;
	return $locations;
}

function nksub_upload_filter( $file ){
	$arr = wp_check_filetype(basename($file['name']));
	$type = $arr['type'];
	$file['name'] = 'nksub_pic.' . str_replace('image/', '', $type);
	return $file;
}

function nksub_debug_to_console($data) {
	if(is_array($data) || is_object($data))
	{
		echo("<script>console.log('PHP: ".json_encode($data)."');</script>");
	} else {
		echo("<script>console.log('PHP: ".$data."');</script>");
	}
}

function nksub_get_subscribers_csv() {
	global $wpdb;

	$id = $_GET['form_id'];

	$subscribers_table_name = $wpdb->prefix . 'nksubsubscribers';

	try {
		if(isset($id) && !empty($id)){
			$sql = $wpdb->prepare("SELECT * FROM {$subscribers_table_name} WHERE form_id = %d", $id);
			$query = $wpdb->get_results ($sql);
		}else{
			$query = $wpdb->get_results ("SELECT * FROM {$subscribers_table_name}");
		}

		$result_csv = "ID, Time, First Name, Last Name, Email, City, Form used, Provider used, Provider response successful?\n";


		foreach($query as $row) {
			$additional_fields = unserialize($row -> additional_fields_hash);

			$values = array(
				$row -> id,
				$row -> time,
				$row -> first_name,
				$row -> last_name,
				$row -> email,
				$additional_fields -> city,
				$row -> form_id,
				$row -> provider_used,
				$row -> provider_response_positive ? 'YES' : 'NO'
			);
			# additional_fields_hash - not used for now, later it should get uniq fields and add them to head row
			$result_csv .= join(', ', $values)."\n";
		}

		header( 'Content-Type: text/csv' );
		header( 'Content-Disposition: attachment;filename=subscribers.csv');

		print($result_csv);
	}catch (Exception $e){
		print 'Failed to generate CSV'; #$e;
	}

}